#include<string.h>
#include<stdlib.h>
#include<string>
#include<sstream>
#include "measure.hh"

std::string toString (int aNumber)
{
  std::stringstream lStringStream;
  lStringStream << aNumber;
  return lStringStream.str();
}

// fill int and string array with random numbers from [0,1000]
void fillData(int* aArray, std::string* aStringArray, int aSize) {
  for (int i = 0; i < aSize; ++i) {
    aArray[i] = rand() % 1001;
    aStringArray[i] = toString(aArray[i]);
  }
}


void printData(int* aArray, std::string* aStringArray, int aSize) {
  for (int i = 0; i < aSize; ++i) {
    std::cout << "aArray[" << i << "]: " << aArray[i] << std::endl;
  }
  for (int i = 0; i < aSize; ++i) {
    std::cout << "aStringArray[" << i << "]: " << aStringArray[i] << std::endl;
  }
}

// cheap predicates with single ampersand
int singleAmpCheap(int* aInput, int* aOutput, int aSize, int aLower, int aUpper) {
  int j = 0;
  for (int i = 0; i < aSize; ++i) {
    if ((aLower <= aInput[i]) & (aInput[i] <= aUpper)) {
      aOutput[j++] = aInput[i];
    }
  }
  std::cout << __FUNCTION__ << ": j: " << j << std::endl;
  return j;
}

// cheap predicates with double ampersand
int doubleAmpCheap(int* aInput, int* aOutput, int aSize, int aLower, int aUpper) {
  int j = 0;
  for (int i = 0; i < aSize; ++i) {
    if ((aLower <= aInput[i]) && (aInput[i] <= aUpper)) {
      aOutput[j++] = aInput[i];
    }
  }
  std::cout << __FUNCTION__ << ": j: " << j << std::endl;
  return j;
}

// expensive predicates with single ampersand
int singleAmpExp(std::string* aInput, std::string* aOutput, int aSize, std::string& aLower, std::string& aUpper) {
  int j = 0;
  for (int i = 0; i < aSize; ++i) {
    if ((aLower <= aInput[i]) & (aInput[i] <= aUpper)) {
      aOutput[j++] = aInput[i];
    }
  }
  std::cout << __FUNCTION__ << ": j: " << j << std::endl;
  return j;
}

// expensive predicates with double ampersand
int doubleAmpExp(std::string* aInput, std::string* aOutput, int aSize, std::string& aLower, std::string& aUpper) {
  int j = 0;
  for (int i = 0; i < aSize; ++i) {
    if ((aLower <= aInput[i]) && (aInput[i] <= aUpper)) {
      aOutput[j++] = aInput[i];
    }
  }
  std::cout << __FUNCTION__ << ": j: " << j << std::endl;
  return j;
}

int main(int argc, char* argv[]) {
  bool lSingle = false;
  int lSize = 100000000;
  // lower and upper bounds for range predicates
  int lLower = 250;
  int lUpper = 750;
  std::string lLowerString;
  std::string lUpperString;
  bool print = false;

  if (argc > 1) {
    for (int i = 1; i < argc; ++i) {
      if (!strcmp(argv[i], "-s")) {
        lSingle = true;
      }   

      if (!strcmp(argv[i], "-n")) {
        lSize = atol(argv[++i]);
      }
      
      if (!strcmp(argv[i], "-l")) {
        lLower = atol(argv[++i]);
        lLowerString = toString(lLower);
      }   

      if(!strcmp(argv[i], "-u")) {
        lUpper = atol(argv[++i]);
        lUpperString = toString(lUpper);
      }   

      if(!strcmp(argv[i], "-p")) {
        print = true;
      }   
    }   
  }

  // allocate input and output arrays
  int* lInputCheap = new int[lSize];
  int* lOutputCheap = new int[lSize];
  std::string* lInputExp = new std::string[lSize];
  std::string* lOutputExp = new std::string[lSize];
  //fill input arrays
  fillData(lInputCheap, lInputExp, lSize);

  if (print) {
    printData(lInputCheap, lInputExp, lSize);
  }

  // number of qualifying entries
  int cardSingleAmpCheap = 0;
  int cardSingleAmpExp = 0;
  int cardDoubleAmpCheap = 0;
  int cardDoubleAmpExp = 0;

  Measure lMeasureCheap;
  Measure lMeasureExp;
  // measure runtime for single ampersand w/ cheap and expensive predicates
  if (lSingle) {
    lMeasureCheap.start();
    cardSingleAmpCheap = singleAmpCheap(lInputCheap, lOutputCheap, lSize, lLower, lUpper);
    lMeasureCheap.stop();
    lMeasureExp.start();
    cardSingleAmpExp = singleAmpExp(lInputExp, lOutputExp, lSize, lLowerString, lUpperString);
    lMeasureExp.stop();
  }

  // measure runtime for double ampersand w/ cheap and expensive predicates
  else {
    lMeasureCheap.start();
    cardDoubleAmpCheap = doubleAmpCheap(lInputCheap, lOutputCheap, lSize, lLower, lUpper);
    lMeasureCheap.stop();
    lMeasureExp.start();
    cardDoubleAmpExp = doubleAmpExp(lInputExp, lOutputExp, lSize, lLowerString, lUpperString);
    lMeasureExp.stop();
  }

  // print runtime
  std::cout << (lSingle?"&: ":"&&: ")  << std::endl;
  std::cout << "Cheap: " << lMeasureCheap.mCpuTime() << "s, Exp: " <<  lMeasureExp.mCpuTime() << "s" << std::endl;

  // print result cardinalities
  if (lSingle) {
    std::cout << "cardinality cheap predicates: " << cardSingleAmpCheap << std::endl;
    std::cout << "cardinality exp predicates: "   << cardSingleAmpExp << std::endl;
  } else {
    std::cout << "cardinality cheap predicates: " << cardDoubleAmpCheap << std::endl;
    std::cout << "cardinality exp predicates: "   << cardDoubleAmpExp << std::endl;
  }

  // release memory
  delete[] lInputCheap;
  delete[] lOutputCheap;
  delete[] lInputExp;
  delete[] lOutputExp;

  return 0;
}
