#include<string.h>
#include<stdlib.h>
#include "measure.hh"

// fill int array with random numbers from [0,100]
void fillData(int* aArray, int aSize) {
  for (int i = 0; i < aSize; ++i) {
    aArray[i] = rand() % 101;   
  }
}

// select array elements < aValue w/ branching
void selectBranch(int* aInput, int* aOutput, int aSize, int aValue) {
   int j = 0;
   for (int i = 0; i < aSize; ++i) {
     if (aInput[i] < aValue) {
       aOutput[j++] = aInput[i];
     }
   }
}

// select array elements < aValue w/o branching (predicated)
void selectPredicated(int* aInput, int* aOutput, int aSize, int aValue) {
   int j = 0;
   for (int i = 0; i < aSize; ++i) {
     aOutput[j] = aInput[i];
     j += (aInput[i] < aValue);
   }
}

int main(int argc, char* argv[]) {
  int lSize = 1000000;
  // comparison value for predicate 
  int lValue = 50;
  // run code w/ or w/o branch?
  bool lPredicated = false;

  if (argc > 1) {
    for (int i = 1; i < argc; ++i) {

      if (!strcmp(argv[i], "-p")) {
        lPredicated = true;
      }

      if (!strcmp(argv[i], "-n")) {
        lSize = atol(argv[++i]);
      }

      if (!strcmp(argv[i], "-v")) {
        lValue = atol(argv[++i]);
      }
    }   
  }

  // allocate input and output arrays
  int* lInput = new int[lSize];
  int* lOutput = new int[lSize];
  //fill input array
  fillData(lInput, lSize);

  Measure lMeasure;
  if (lPredicated) {
    lMeasure.start();
    selectPredicated(lInput, lOutput, lSize, lValue);
    lMeasure.stop();
  }
  else {
    lMeasure.start();
    selectBranch(lInput, lOutput, lSize, lValue);
    lMeasure.stop();
  }
  // print runtime
  std::cout << "Selectivity " << (float)lValue/100 << ": " << lMeasure.mCpuTime() << "s" << std::endl;

  // release memory
  delete[] lInput;
  delete[] lOutput;

  return 0;
}
